(ns znuli.core (:import GifSequenceWriter) (:gen-class))

(use 'znuli.gif)
(import '(java.awt Robot Toolkit)
'(java.awt BorderLayout Frame Cursor)
'(java.awt.event ActionListener)
'(java.util Calendar)
'(java.text SimpleDateFormat)
'(javax.imageio ImageIO ImageWriter ImageWriteParam)
'(java.io File)
'(javax.swing
JButton JFrame ImageIcon JLabel
JPanel JProgressBar Timer
JSpinner SpinnerNumberModel))

(defn get-icon [icon]
  (clojure.java.io/resource icon))

(def menu (JFrame. "Znuli 0.1.2"))
(doto menu
  (.setResizable false)
  (.setAlwaysOnTop true))

(def progressbar (JProgressBar. 0, 3))
(def selection-btn (JButton. (ImageIcon. (get-icon "search.png"))))

(def selection (JFrame. "Select area"))
(doto selection
  (.setSize 300 300)
  (.setLocation 350 100)
  (.setAlwaysOnTop true)
  (.add selection-btn)
  (.setIconImage (.. Toolkit getDefaultToolkit (getImage (get-icon "search.png")))))

(def pictures (ref '()))

(defn screenshot []
  (let [filename (str "img" (.getTime (java.util.Date.)) ".png")
      rbt (Robot.)
      tk (Toolkit/getDefaultToolkit)
      ss (.getScreenSize tk)]
  (.setVisible selection false)
  (.setVisible menu false)
  (.delay rbt 200)
  (dosync (ref-set pictures (cons filename (deref pictures)))) 
  (.setValue progressbar (mod (count (deref pictures)) 4))
  (.pack menu)
  (ImageIO/write
	(.createScreenCapture rbt (.getBounds selection))
	"png"
	(File. filename))
  (.setVisible menu true)))

(def size-btn (JButton.  (ImageIcon. (get-icon "search.png"))))
(.setToolTipText size-btn "Select area to photograph for gif")
(def screenshot-btn (JButton.  (ImageIcon. (get-icon "add.png"))))
(.setToolTipText screenshot-btn "Add 1 frame to gif")
(def save-btn (JButton.  (ImageIcon. (get-icon "save.png"))))
(.setToolTipText save-btn "Export gif animation to file")
(def clear-btn (JButton. (ImageIcon. (get-icon "clear.png"))))
(.setToolTipText clear-btn "New gif animation (Reset)")
(def rec-btn (JButton.  (ImageIcon. (get-icon "record.png"))))
(.setToolTipText rec-btn "Record video for gif")
(def speed-btn (JSpinner. (SpinnerNumberModel. 15 0 99 1)))
(.setToolTipText speed-btn "Speed (Frames per second)")
(def time-btn (JSpinner. (SpinnerNumberModel. 10 1 99 1)))
(.setToolTipText time-btn "Duration of video-gif (Seconds)")
(def help-btn (JButton.  (ImageIcon. (get-icon "help.png"))))
(.setToolTipText help-btn "You need help?")

(def speed-panel (JPanel.))
(doto speed-panel
  (.add (JLabel. (ImageIcon. (get-icon "forward.png"))))
  (.add speed-btn))

(def gif-panel (JPanel.))
(doto gif-panel
  (.add size-btn)
  (.add screenshot-btn)
  (.add save-btn)
  (.add clear-btn))

(def settings-panel (JPanel.))
(doto settings-panel
  (.add speed-panel)
  (.add progressbar))

(def advanced-panel (JPanel.))
(doto advanced-panel
  (.add (JLabel. (ImageIcon. (get-icon "new.png"))))
  (.add time-btn)
  (.add rec-btn)
  (.add help-btn))

(def select-size
  (proxy [ActionListener] [] (actionPerformed [e]
    (.setVisible selection true))))
(.addActionListener size-btn select-size)

(def clear
  (proxy [ActionListener] [] (actionPerformed [e]
  (dosync (ref-set pictures '())) 
  (.setValue progressbar 0))))
(.addActionListener clear-btn clear)


(defn fps-to-mspf [fps]
  (cond
    (= fps 0) 1
    true (int (* (/ 1 fps) 1000))))

(defn rec [frames]
  (let [filename (str "img" (int (rand 9999)) (.getTime (java.util.Date.)) ".png")
      rbt (Robot.)
      tk (Toolkit/getDefaultToolkit)
      ss (.getScreenSize tk)]
  (.setVisible selection false)
  (cond
    (zero? frames) (print "record done")
    true ((.delay rbt (/ (fps-to-mspf (.getValue speed-btn)) 2))
  (dosync (ref-set pictures (cons filename (deref pictures)))) 
  (ImageIO/write
	(.createScreenCapture rbt (.getBounds selection))
	"png"
	(File. filename))
    (rec (dec frames))))))

(.addActionListener screenshot-btn
(proxy [ActionListener] [] (actionPerformed [e]
(screenshot))))

(.addActionListener rec-btn
(proxy [ActionListener] [] (actionPerformed [e]
(rec (* (.getValue speed-btn) (.getValue time-btn))))))


(def save
  (proxy [ActionListener] [] (actionPerformed [e]
  (.setIndeterminate progressbar true)
  (cond
    (not (empty? (deref pictures)))
      (create-gif (deref pictures) (fps-to-mspf (.getValue speed-btn)) true))
  (.setIndeterminate progressbar false))))
(.addActionListener save-btn save)

(def set-size
	(proxy [ActionListener] [] (actionPerformed [e]
		(.setVisible selection false))))
(.addActionListener selection-btn set-size)


(doto menu
  (.add gif-panel BorderLayout/NORTH)
  (.add settings-panel BorderLayout/CENTER)
  (.add advanced-panel BorderLayout/SOUTH)
  (.pack)
  (.setLocation 600 0)
  (.setIconImage (.. Toolkit getDefaultToolkit (getImage (get-icon "znuli.png"))))
  (.setDefaultCloseOperation JFrame/EXIT_ON_CLOSE))


(defn -main
  [& args]
(.setVisible menu true)  
  )

